/*******************************************************************************
* File Name: Mix.c  
* Version 2.0
*
* Description:
*  This file provides the source code to the API for the MIXER Component.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "Mix.h"

#if (!CY_PSOC5A)
    #if (CYDEV_VARIABLE_VDDA == 1u)
        #include "CyScBoostClk.h"
    #endif /* (CYDEV_VARIABLE_VDDA == 1u) */
#endif /* (!CY_PSOC5A) */

#if (Mix_LO_SOURCE == Mix_LO_SOURCE_INTERNAL)
    #include "Mix_aclk.h"
#endif /* Mix_LO_SOURCE */

/* Fixed configuration of SC Block only performed once */
uint8 Mix_initVar = 0u;

/* Only for PSoC5A */
#if (CY_PSOC5A)
    static Mix_backupStruct Mix_P5backup;
#endif /* CY_PSOC5A */


/*******************************************************************************
* Function Name: Mix_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  Mix_Start().
*
* Parameters:  
*  void
*
* Return: 
*  void
*
*******************************************************************************/
void Mix_Init(void) 
{
    float LOf = Mix_LO_CLOCK_FREQ;
    
    /* Configure SC Block based on selected Mixer type */
    /* Continuous Time (up) mixer */
    #if (Mix_MIXER_TYPE == Mix_CTMIXER) 
        /* SC Block mode -  - SCxx_CR0[3:1] */
        Mix_CR0_REG = Mix_MODE_CTMIXER;   

        /* SC Block CR1 */ 
        Mix_CR1_REG = (Mix_COMP_4P35PF  |
                                Mix_DIV2_DISABLE |
                                Mix_GAIN_0DB);                                 

        /* 
            Set SC Block resistor values based on Local Oscillator being above 
            or below 100kHz - controled by r20_40 (Rmix) bit for CT mode mixer
            and SC Block
        */

        if (LOf < Mix_LO_CLOCK_FREQ_100_KHz)
        {
            Mix_CR2_REG = (Mix_BIAS_LOW |
										Mix_REDC_01 |
										Mix_GNDVREF_DI |
										Mix_R20_40B_40K);
        }
        else
        {
            Mix_CR2_REG = (Mix_BIAS_LOW |
										Mix_REDC_01 |
										Mix_GNDVREF_DI |
										Mix_R20_40B_20K);
        }
    #else
        /* Discrete Time (down) mixer */
        /* SC Block mode -  - SCxx_CR0[3:1] */
        Mix_CR0_REG = Mix_MODE_DTMIXER;       

        /* SC Block CR1 */ 
        Mix_CR1_REG = (Mix_COMP_4P35PF  |
                                Mix_DIV2_ENABLE |
                                Mix_GAIN_0DB);

        /* 
            Set SC Block resistor values based on Local Oscillator beign above 
            or below 100kHz - set r20_40 (input resitance) and rval (feedback) 
            resistor values seperately for DT mode and SC Block
        */
        if (LOf < Mix_LO_CLOCK_FREQ_100_KHz)
        {
            Mix_CR2_REG = (Mix_BIAS_LOW |
										Mix_GNDVREF_DI |
										Mix_R20_40B_40K |
										Mix_RVAL_40K); 
        }
        else
        {
            Mix_CR2_REG = (Mix_BIAS_LOW |
										Mix_GNDVREF_DI |
										Mix_R20_40B_20K |
										Mix_RVAL_20K);
        }
    #endif /* end if - Mixer Type */

    /* Enable SC Block clocking */
    Mix_CLK_REG |= Mix_CLK_EN;

    /* Set default power */
    Mix_SetPower(Mix_INIT_POWER);
    
    /* Set 50 % Duty cycle for LO clock if LO source is internal */
    #if(Mix_LO_SOURCE == Mix_LO_SOURCE_INTERNAL)
        Mix_aclk_SetMode(CYCLK_DUTY);
    #endif /* Mix_LO_SOURCE == Mix_LO_SOURCE_INTERNAL */

}


/*******************************************************************************
* Function Name: Mix_Enable
********************************************************************************
*  
* Summary: 
*  Enables the Mixer block operation
*
* Parameters:  
*  void
*
* Return: 
*  void
*
*******************************************************************************/
void Mix_Enable(void) 
{
    /* This is to restore the value of register CR1 and CR2 which is saved 
      in prior to the modifications in stop() API */
    #if (CY_PSOC5A)
        if(Mix_P5backup.enableState == 1u)
        {
            Mix_CR1_REG = Mix_P5backup.scCR1Reg;
            Mix_CR2_REG = Mix_P5backup.scCR2Reg;
            Mix_P5backup.enableState = 0u;
        }
    #endif /* CY_PSOC5A */
    
    /* Enable power to SC block in active mode */
    Mix_PM_ACT_CFG_REG |= Mix_ACT_PWR_EN;

    /* Enable power to SC block in Alternative active mode */
    Mix_PM_STBY_CFG_REG |= Mix_STBY_PWR_EN;
    
    /* Enable LO_Internal clock if LO source is choosen as internal */
    #if(Mix_LO_SOURCE == Mix_LO_SOURCE_INTERNAL)
        /* Enable power to the LO clock */
        Mix_PWRMGR_ACLK_REG |= Mix_ACT_PWR_ACLK_EN;        
        Mix_STBY_PWRMGR_ACLK_REG |= Mix_STBY_PWR_ACLK_EN;
        /* Enable the clock */
        Mix_aclk_Enable();        
    #endif /* Mix_LO_SOURCE == Mix_LO_SOURCE_INTERNAL */

    Mix_PUMP_CR1_REG |= Mix_PUMP_CR1_SC_CLKSEL;
    
    #if (!CY_PSOC5A)
        #if (CYDEV_VARIABLE_VDDA == 1u)
            if(CyScPumpEnabled == 1u)
            {
                Mix_BSTCLK_REG &= (uint8)(~Mix_BST_CLK_INDEX_MASK);
                Mix_BSTCLK_REG |= Mix_BST_CLK_EN | CyScBoostClk__INDEX;
                Mix_SC_MISC_REG |= Mix_PUMP_FORCE;
                CyScBoostClk_Start();
            }
            else
            {
                Mix_BSTCLK_REG &= (uint8)(~Mix_BST_CLK_EN);
                Mix_SC_MISC_REG &= (uint8)(~Mix_PUMP_FORCE);
            }
        #endif /* (CYDEV_VARIABLE_VDDA == 1u) */
    #endif /* (!CY_PSOC5A) */
}


/*******************************************************************************
* Function Name: Mix_Start
********************************************************************************
*
* Summary:
*  The start function initializes the SC block control registers based on the
*  Mixer type selected and and enables power to the SC block
*
* Parameters:  
*  void
*
* Return: 
*  void 
*
* Global variables:
*  Mix_initVar:  Used to check the initial configuration,
*  modified when this function is called for the first time.
*
* Theory: 
*  Full initialization of the SC Block configuration registers is only perfomed 
*  the first time the routine executes - global variable 
*  Mix_initVar is used indicate that the static configuration has 
*  been completed.
*
*******************************************************************************/
void Mix_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(Mix_initVar == 0u)
    {
        Mix_Init();
        Mix_initVar = 1u;
    }
    /* Enable the power to the block */
    Mix_Enable();
    
}


/*******************************************************************************
* Function Name: Mix_Stop
********************************************************************************
*
* Summary:
*  Disables power to SC block.
*
* Parameters:  
*  void 
*
* Return: 
*  void 
*
*******************************************************************************/
void Mix_Stop(void) 
{       
    /* Disble power to the Amp in Active mode template */
    Mix_PM_ACT_CFG_REG &= (uint8)(~Mix_ACT_PWR_EN);

    /* Disble power to the Amp in Alternative Active mode template */
    Mix_PM_STBY_CFG_REG &= (uint8)(~Mix_STBY_PWR_EN);
    
    #if (!CY_PSOC5A)
        #if (CYDEV_VARIABLE_VDDA == 1u)
            Mix_BSTCLK_REG &= (uint8)(~Mix_BST_CLK_EN);
            /* Disable pumps only if there aren't any SC block in use */
            if ((Mix_PM_ACT_CFG_REG & Mix_PM_ACT_CFG_MASK) == 0u)
            {
                Mix_SC_MISC_REG &= (uint8)(~Mix_PUMP_FORCE);
                Mix_PUMP_CR1_REG &= (uint8)(~Mix_PUMP_CR1_SC_CLKSEL);
                CyScBoostClk_Stop();
            }
        #endif /* CYDEV_VARIABLE_VDDA == 1u */
    #endif /* (CY_PSOC3 || CY_PSOC5LP) */
    
    /* This sets Sample and hold in zero current mode and output routes are valid */
    #if (CY_PSOC5A)
        Mix_P5backup.scCR1Reg = Mix_CR1_REG;
        Mix_P5backup.scCR2Reg = Mix_CR2_REG;
        Mix_CR1_REG = 0x00u;
        Mix_CR2_REG = 0x00u;
        Mix_P5backup.enableState = 1u;
    #endif /* CY_PSOC5A */  
    
    /* Disable aclk */
    #if(Mix_LO_SOURCE == Mix_LO_SOURCE_INTERNAL)
        /* Disable power to clock */
        Mix_PWRMGR_ACLK_REG &= (uint8)(~Mix_ACT_PWR_ACLK_EN);
        Mix_STBY_PWRMGR_ACLK_REG &= (uint8)(~Mix_STBY_PWR_ACLK_EN);
        /* Disable the clock */
        Mix_aclk_Disable();
    #endif /* Mix_LO_SOURCE == Mix_LO_SOURCE_INTERNAL */
}


/*******************************************************************************
* Function Name: Mix_SetPower
********************************************************************************
*
* Summary:
*  Set the drive power of the MIXER
*
* Parameters:  
*  power:  Sets power level between (0) and (3) high power
*
* Return: 
*  void 
*
*******************************************************************************/
void Mix_SetPower(uint8 power) 
{
    uint8 tmpCR;

    /* Sets drive bits in SC Block Control Register 1:  SCxx_CR[1:0] */    
    tmpCR = Mix_CR1_REG & (uint8)(~Mix_DRIVE_MASK);
    tmpCR |= (power & Mix_DRIVE_MASK);
    Mix_CR1_REG = tmpCR;  
}


/* [] END OF FILE */
